/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 * \brief This file contains API for DIGI POT Click driver.
 *
 * \addtogroup digipot DIGI POT Click Driver
 * @{
 */
// ----------------------------------------------------------------------------

#ifndef DIGIPOT_H
#define DIGIPOT_H

#include "drv_digital_out.h"
#include "drv_spi_master.h"

// -------------------------------------------------------------- PUBLIC MACROS 
/**
 * \defgroup macros Macros
 * \{
 */

/**
 * \defgroup map_mikrobus MikroBUS
 * \{
 */

#define DIGIPOT_MAP_MIKROBUS( cfg, mikrobus ) \
   cfg.miso  = MIKROBUS( mikrobus, MIKROBUS_MISO ); \
   cfg.mosi  = MIKROBUS( mikrobus, MIKROBUS_MOSI ); \
   cfg.sck   = MIKROBUS( mikrobus, MIKROBUS_SCK ); \
   cfg.cs    = MIKROBUS( mikrobus, MIKROBUS_CS ); 
/** \} */

/**
 * \defgroup error_code Error Code
 * \{
 */
#define DIGIPOT_RETVAL  uint8_t

#define DIGIPOT_OK           0x00
#define DIGIPOT_INIT_ERROR   0xFF
/** \} */

/**
 * \defgroup digipot_cmd DIGIPOT Command
 * \{
 */
#define DIGIPOT_VOLATILE_WIPER_0             0x00
#define DIGIPOT_VOLATILE_WIPER_1             0x01
#define DIGIPOT_NON_VOLATILE_WIPER_0         0x02
#define DIGIPOT_NON_VOLATILE_WIPER_1         0x03
#define DIGIPOT_VOLATILE_TCON_REGISTER       0x04
#define DIGIPOT_STATUS_REGISTER              0x05
/** \} */

/**
 * \defgroup eeprom_data EEPROM data
 * \{
 */
#define DIGIPOT_DATA_EEPROM_6                0x06
#define DIGIPOT_DATA_EEPROM_7                0x07
#define DIGIPOT_DATA_EEPROM_8                0x08
#define DIGIPOT_DATA_EEPROM_9                0x09
#define DIGIPOT_DATA_EEPROM_A                0x0A
#define DIGIPOT_DATA_EEPROM_B                0x0B
#define DIGIPOT_DATA_EEPROM_C                0x0C
#define DIGIPOT_DATA_EEPROM_D                0x0D
#define DIGIPOT_DATA_EEPROM_E                0x0E
#define DIGIPOT_DATA_EEPROM_F                0x0F
#define DIGIPOT_DATA_WRITE_CMD               0x20
/** \} */

/** \} */ // End group macro 
// --------------------------------------------------------------- PUBLIC TYPES
/**
 * \defgroup type Types
 * \{
 */

/**
 * @brief Click ctx object definition.
 */
typedef struct
{
    // Modules 

    spi_master_t spi;

} digipot_t;

/**
 * @brief Click configuration structure definition.
 */
typedef struct
{
    // Communication gpio pins 

    pin_name_t miso;
    pin_name_t mosi;
    pin_name_t sck;
    pin_name_t cs;

    // static variable 

    hal_spi_speed_t spi_speed;
    hal_spi_mode_t spi_mode;

} digipot_cfg_t;

/** \} */ // End types group
// ----------------------------------------------- PUBLIC FUNCTION DECLARATIONS

/**
 * \defgroup public_function Public function
 * \{
 */
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief Config Object Initialization function.
 *
 * @param cfg  Click configuration structure.
 *
 * @description This function initializes click configuration structure to init state.
 * @note All used pins will be set to unconnected state.
 */
void digipot_cfg_setup ( digipot_cfg_t *cfg );

/**
 * @brief Initialization function.
 * @param digipot Click object.
 * @param cfg Click configuration structure.
 * 
 * @description This function initializes all necessary pins and peripherals used for this click.
 */
DIGIPOT_RETVAL digipot_init ( digipot_t *ctx, digipot_cfg_t *cfg );

/**
 * @brief Generic transfer function.
 *
 * @param ctx          Click object.
 * @param block        Transfer block structure.
 *
 * @description Generic SPI transfer, for sending and receiving packages
 */
void digipot_generic_transfer ( digipot_t *ctx, spi_master_transfer_data_t *block );

/**
 * @brief Set wiper position
 *
 * @param ctx          Click object.
 * @param wiper_pos    uint8_t wiper positions data.
 *
 * @description The function sets 8-bit wiper positions data.
 */
void digipot_set_wiper_positions ( digipot_t *ctx, uint8_t wiper_pos );

/**
 * @brief Convert ADC value to volatage
 *
 * @param adc_val      10-bit ADC value.
 * @param wiper_pos    float voltage reference: 3.3V or 5V.
 *
 * @return             float voltage ( 0V - 3.3V ) or ( 0V - 5.0V ).
 * 
 * @description The function convert 10-bit ADC value 
 * to volatage reference.
 */
float digipot_convert_output ( uint16_t adc_val, float v_ref );

#ifdef __cplusplus
}
#endif
#endif  // _DIGIPOT_H_

/** \} */ // End public_function group
/// \}    // End click Driver group  
/*! @} */
// ------------------------------------------------------------------------- END
