/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "digipot.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define DIGIPOT_DUMMY 0

#define DIGIPOT_VREF_3V3                     3.30
#define DIGIPOT_VREF_5V                      5.00
#define DIGIPOT_CON_CALC_VAL                 255.00

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void digipot_cfg_setup ( digipot_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

DIGIPOT_RETVAL digipot_init ( digipot_t *ctx, digipot_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return DIGIPOT_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, DIGIPOT_DUMMY );

    return DIGIPOT_OK;
}

void digipot_generic_transfer ( digipot_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);    
}

// Set 8-bit wiper positions
void digipot_set_wiper_positions ( digipot_t *ctx, uint8_t wiper_pos )
{
    spi_master_transfer_data_t block;
    uint8_t write_buf[ 4 ];

    write_buf[ 0 ] = DIGIPOT_VOLATILE_WIPER_0;
    write_buf[ 1 ] = wiper_pos;
    write_buf[ 2 ] = DIGIPOT_DATA_WRITE_CMD;
    write_buf[ 3 ] = wiper_pos;
    
    block.tx_buffer = write_buf;
    block.tx_length = 4;
    block.rx_buffer = 0;
    block.rx_length = 0;

    digipot_generic_transfer( ctx, &block ); 
}

// Convert ADC value to volatage
float digipot_convert_output ( uint16_t adc_val, float v_ref )
{
    float result;

    result = ( float ) adc_val;
    result *= v_ref;
    result/= DIGIPOT_CON_CALC_VAL;

    return result;
}

// ------------------------------------------------------------------------- END

